use serde::Serialize;

/// This is a static configuration object.
/// It might be changed in the future to have a dynamic counterpart.
///
/// ## Example:
/// Creating a simple required and readonly text input field with a label:
///
/// ```ignore
/// let my_field = Field::widget("/admin/widgets/input_text.jinja")
///     .labelled("Username")
///     .required()
///     .readonly();
/// ```
#[derive(Debug, Serialize, Clone, Copy)]
pub struct Widget {
    pub widget: &'static str,
    pub label: Option<&'static str>,
    #[serde(rename = "type")]
    pub field_type: &'static str,
    pub required: bool,
    pub readonly: bool,
    pub options: &'static [(&'static str, &'static str)],
}

impl Widget {
    pub fn widget(widget: &'static str) -> Self {
        Widget {
            widget: widget,
            label: None,
            field_type: "text",
            required: false,
            readonly: false,
            options: &[],
        }
    }

    pub fn default() -> Self {
        Self::widget("/admin/widgets/input_text.jinja")
    }

    pub fn textarea() -> Self {
        Self::widget("/admin/widgets/input_textarea.jinja")
    }

    pub fn checkbox() -> Self {
        Self::widget("/admin/widgets/checkbox_toggle.jinja")
    }

    pub fn required(mut self) -> Self {
        self.required = true;
        self
    }

    pub fn readonly(mut self) -> Self {
        self.readonly = true;
        self
    }

    pub fn labeled(mut self, label: &'static str) -> Self {
        self.label = Some(label);
        self
    }

    pub fn as_password(mut self) -> Self {
        self.field_type = "password";
        self
    }

    pub fn as_hidden(mut self) -> Self {
        self.field_type = "hidden";
        self
    }

    pub fn options(mut self, options: &'static [(&'static str, &'static str)]) -> Self {
        self.options = options;
        self
    }
}
